package com.eyeverify.cameraplay;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import android.util.Log;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.media.MediaRecorder;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.app.Activity;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;

public class CameraPreviewActivity extends Activity {
	private static final String TAG = "CameraPreviewActivity";
	
	private Camera mCamera;
	private CameraSurfaceView mPreview;
	private MediaRecorder mMediaRecorder;
	
	public static final int MEDIA_TYPE_IMAGE = 1;
	public static final int MEDIA_TYPE_VIDEO = 2;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.camera_layout);
		String whichCamera = getIntent().getStringExtra("CameraID");
		
		Log.d(TAG, "WhichCamera = " + whichCamera);
		
		if (whichCamera.equals("back")) {
			mCamera = openBackFacingCameraGingerbread();
		} else {
			mCamera = openFrontFacingCameraGingerbread();
			Camera.Parameters parameters = mCamera.getParameters();
			List<Size> cameraSizes = parameters.getSupportedPictureSizes();
			
//			for (Size size : cameraSizes) {
//				Log.d(TAG, "Height: " + size.height + " Width: " + size.width);
//			}
//			parameters.setPictureSize(1392, 1392);
			mCamera.setParameters(parameters);
		}
		
//		final PictureCallback mPicture = new PictureCallback() {
//
//		    @Override
//		    public void onPictureTaken(byte[] data, Camera camera) {
//
//		    	Log.d(TAG, "On Picture Taken");
////		        File pictureFile = getOutputMediaFile(MEDIA_TYPE_IMAGE);
////		        if (pictureFile == null){
////		            Log.d(TAG, "Error creating media file, check storage permissions: ");
////		            return;
////		        }
////
////		        try {
////		        	Log.d(TAG, "Will try opening filestream");
////		            FileOutputStream fos = new FileOutputStream(pictureFile);
////		            fos.write(data);
////		            Log.d(TAG, "Will try closing filestream");
////		            fos.close();
////		        } catch (FileNotFoundException e) {
////		            Log.d(TAG, "File not found: " + e.getMessage());
////		        } catch (IOException e) {
////		            Log.d(TAG, "Error accessing file: " + e.getMessage());
////		        }
//		    }
//		};
		
		TextView txtFrameRate = (TextView) findViewById(R.id.txtFrameRate);
		
		mPreview = new CameraSurfaceView(this, mCamera, txtFrameRate/*, mPicture*/);
		FrameLayout preview = (FrameLayout) findViewById(R.id.camera_preview);
		preview.addView(mPreview);
		
		Button captureButton = (Button) findViewById(R.id.button_capture);
		captureButton.setOnClickListener(
		    new View.OnClickListener() {
		        @Override
		        public void onClick(View v) {
		        	Log.d(TAG, "Button Click");
		            // get an image from the camera
		            mCamera.takePicture(null, null, mPreview);
		        }
		    }
		);
	}
	

    @Override
    protected void onPause() {
        super.onPause();
        releaseMediaRecorder();       // if you are using MediaRecorder, release it first
        releaseCamera();              // release the camera immediately on pause event
    }

    private void releaseMediaRecorder(){
        if (mMediaRecorder != null) {
            mMediaRecorder.reset();   // clear recorder configuration
            mMediaRecorder.release(); // release the recorder object
            mMediaRecorder = null;
            mCamera.lock();           // lock camera for later use
        }
    }

    private void releaseCamera(){
        if (mCamera != null){
            mCamera.release();        // release the camera for other applications
            mCamera = null;
        }
    }
    
    /** Create a file Uri for saving an image or video */
    private static Uri getOutputMediaFileUri(int type){
          return Uri.fromFile(getOutputMediaFile(type));
    }

    /** Create a File for saving an image or video */
    private static File getOutputMediaFile(int type){
        // To be safe, you should check that the SDCard is mounted
        // using Environment.getExternalStorageState() before doing this.

        File mediaStorageDir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES), "cameraplay");
        // This location works best if you want the created images to be shared
        // between applications and persist after your app has been uninstalled.

        // Create the storage directory if it does not exist
        if (! mediaStorageDir.exists()){
            if (! mediaStorageDir.mkdirs()){
                Log.d("MyCameraApp", "failed to create directory");
                return null;
            }
        }

        // Create a media file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        File mediaFile;
        if (type == MEDIA_TYPE_IMAGE){
            mediaFile = new File(mediaStorageDir.getPath() + File.separator +
            "IMG_"+ timeStamp + ".jpg");
        } else if(type == MEDIA_TYPE_VIDEO) {
            mediaFile = new File(mediaStorageDir.getPath() + File.separator +
            "VID_"+ timeStamp + ".mp4");
        } else {
            return null;
        }

        Log.d(TAG, "Filename = " + mediaFile.getAbsolutePath());
        return mediaFile;
    }
    
    private Camera openFrontFacingCameraGingerbread() {
		int cameraCount = 0;
		Camera cam = null;
		Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
		cameraCount = Camera.getNumberOfCameras();
		for (int camIdx = 0; camIdx < cameraCount; camIdx++) {
			Camera.getCameraInfo(camIdx, cameraInfo);
			if (cameraInfo.facing == Camera.CameraInfo.CAMERA_FACING_FRONT) {
				try {
					cam = Camera.open(camIdx);
				} catch (RuntimeException e) {
					Log.e(TAG,
							"Camera failed to open: " + e.getLocalizedMessage());
				}
			}
		}

		return cam;
	}
    
    private Camera openBackFacingCameraGingerbread() {
		int cameraCount = 0;
		Camera cam = null;
		Camera.CameraInfo cameraInfo = new Camera.CameraInfo();
		cameraCount = Camera.getNumberOfCameras();
		for (int camIdx = 0; camIdx < cameraCount; camIdx++) {
			Camera.getCameraInfo(camIdx, cameraInfo);
			if (cameraInfo.facing == Camera.CameraInfo.CAMERA_FACING_BACK) {
				try {
					cam = Camera.open(camIdx);
				} catch (RuntimeException e) {
					Log.e(TAG,
							"Camera failed to open: " + e.getLocalizedMessage());
				}
			}
		}

		return cam;
	}
}
